# MCMC estimation of past effective size from microsatellite data
# V1: August / December 2011  
#
# V2 - July 2013
# (1) producing a more readable .Theta output file
# (2) adding standard deviations of posterior distribution
#      changes involve  functions : () and LogSizeDist(),
#      and internal functions: .LSD() and  .ExtracTheta()
#      and the format of files .Nstat and .Lstat 
# (3) addition of the Tmrca() function to extract the distribution
#      of the Time to the Most Recent Common Ancestor of two alleles
#      from the output of VarEff() (.Batch file). Results include
#      plots and a text file to save trials (.Tmrca file).
# (4) add two MCMC technical parameters, 
#      length of the burnin period and acceptation rate
#
# Three mutation models are proposed: 
#  Single Step Model (S)
#  Two Phase Model   (T)
#  Geometric Model   (G)
#  in the last two cases, an additional parameter C must be given:
#       the rate of mutations with steps of 2 microsatellite motifs (T)
#    or the ratio of the geometric series fixing the distribution of step lengths (G).
#   (this parameter is assumed fixed and it is NOT estimated in the present version)
#
# A mutation rate u (MUTAT parameter) is used to translate standardized units 
# (theta= 4Nu, and reduced time T = Generation * u) into natural values of
# effective size (census of the diploid population), and of time (number of generations).
# However, since the theory makes only use of reduced units, the actual mutation rate
# (parameter MUTAT) is not estimated from data, it must be provided by the user. 
#
# The function Theta() given in a separate file provides a preliminary analysis of data 
# and builds a parameter file to run VarEff.
#
# The present file include the following functions:
#
# VarEff(): the program to run to generate the posterior distribution of effective sizes
#            in the past
#
#           Results: 
#           if the function is called as  Result = VarEff(),
#           Result$Theta  is the set of 3 global theta estimates, and means and standard deviation
#                         of Quadratic deviations and of prior probabilities of simulated states
#           Result$Batch  is the matrix of the simulated states
#
#           Both lists are also written to files  .Theta and .Batch
#
# and five functions allowing the results of VarEff to be analysed and visualized:
#
#    Note that if the parameter MUTAT is not provided in these functions,
#    results are shown using reduced scales (theta's, and G*u units).
#    Any intermediate result is saved under this reduced format.
# 
# Functions NatSizeDist(...) and LogSizeDist(BATCHfile,TMAX,NBT,MUTAT)
# Extracts Ne (or log_10(Ne) ) posterior distributions at a number of past times
# Input:    the name of the parafile.Batch file produced by VarEff
#           the interval of observation of past times TMAX 
#           the number NBT of intervals between observed instants (NBT+1 instants)
#           the MUTAT mutation rate (if set to 0, TMAX must be given in reduced time,
#                                    and results are given in reduced scales)
# Results:  global stats (mean of posterior distributions of Ne at past times)
#              (plot and file parafile.Nstat  or parafile.Lstat)
#           detailed posterior distribution of Ne at the analysed times
#              (file parafile.Ndist or parafile.Ldist)
#
# ! changes in Version 2: 
#    (1)  the .Nstat / .Lstat files include the standard deviation 
#                        of posterior distributions
#         Changes involve functions: NatSizeDist() and LogSizeDist(),
#         and the internal functions: .LSD() and  .ExtracTheta()
#         The format of files .Nstat and .Lstat is changed accordingly, with an additional
#         8th column giving the standard deviation
#
#    (2)  An additional file is generated to provide the list of populations sizes 
#           according to the nbatch simulated states at the specified (NBT+1) instants. 
#           First line: these instants.
#           First column: the numbers of states (0 in the first line)
#              (file parafile.Nsize or parafile.Lsize)
#
# Function NTdist(BATCHfile,TMAX,MUTAT)
# Graphical summary of the 2D joint distribution of N and of T
# Input:    the name of the parafile.Batch file produced by VarEff
#           the interval of observation of past times TMAX 
#           the MUTAT mutation rate (if set to 0, TMAX must be given in reduced time,
#                                    and results are given in reduced scales)
# Results:  the 2D joint distribution of T and log_10( N(T) )
#                  plot using scales as defined by parameter MUTAT 
#                        (reduced scales if MUTAT set to 0)
#              and file parafile.2D using the reduced scales
#
# Function plotNdistrib(NDISTfile)
# Plots posterior distributions of N(T) at given times
# Input:    the name of a parafile.Ndist / parafile.Ldist file produced by the function 
#           NatSizeDist()  or  LogSizeDist().
#           (the choice of scales is made in functions NatSizeDist() and LogSizeDist())
# Results:  plots of the posterior distributions of population size at different
#           past times. Choosing these times and  the range of population sizes
#           to plot is under user's control.
#
# Function Tmrca()   (addition in V2)
# Derives the posterior distribution of  Tmrca, the
#           Time to the Most Recent Common Ancestor of two alleles.
#  Input:   the name of the parafile.Batch file produced by VarEff,
#           the mutation rate, and upper limits of time until which
#           the distribution of Tmrca is calculated. Choosing these times 
#           and is under user's control
#  Outputs: plots of Tmrca and log(Tmrca) distributions.
#           A text file (.Tmrca) to record trials.
# 
#
#############################################################################################
#  Preliminary analysis of data and building a parameter file to run VarEff
#    input : name of the data file  (infile)
#            the number of markers  (parafile)
#            a name for the current parameters (NBLOC)
#
#    output: two files: parafile.theta , three global estimates of theta
#                       parafile.R     , the command to run VarEff
#
Theta <-
function(parafile=NULL,
                         infile=NULL,
                         NBLOC=NULL, 
                         JMAX=NULL,
                         MODEL=NULL,
                         MUTAT=NULL,
                         NBAR=NULL,
                         VARP1=NULL,
                         RHOCORN=NULL,
                         GBAR=NULL,
                         VARP2=NULL,
                         DMAXPLUS=NULL,
                         Diagonale=NULL,
                         NumberBatch=NULL,
                         LengthBatch=NULL,
                         SpaceBatch=NULL,
                         Burnin=NULL,
                         AccRate=NULL)
{
print(" VarEff package (Version 2), function Theta() ",quote="FALSE")
print(" Estimation of global Theta values and choice of parameters to run VarEff()",quote="FALSE")
print(" ",quote="FALSE")
print(" *** Job identification *** ",quote="FALSE")
# If not given, give a name for the job
if(!is.null(parafile)){
print(paste(" Name of the job = ",parafile,sep=""),quote="FALSE")}else{
parafile = readline(" Give a name for this job = ")}
#
print(" *** Data *** ",quote="FALSE")
if(!is.null(infile)){
print (paste(" Name of the data file= ",infile,sep=""),quote="FALSE")}else{
infile = readline(" Name of the data file= ")}
if(!is.null(NBLOC)){
print (paste(" Number of microsatellite markers= ",NBLOC,sep=""),quote="FALSE")}else{ 
NBLOC = as.numeric(readline(" Number of microsatellite markers= "))}
# Read the data from infile
#
##############################################################################
#
# read data and extract allele frequencies
#           skipping loci with 0 sample size
#
# input: infile, data file
#          NBLOC, number of markers
#
# output : NbMaxAll, PkMoyTout, VarCovTout
#          and the effective number of loci : NELOC
#
.DataExtract <- function(infile,NBLOC)
{
# Read the data file
#
TOUT <- scan(infile)
#
# First step: derive the maximum number of  alleles "NbMaxAll" 
#             from one or several samples of the population: 
#
NbMaxAll = 1
posit = 0
#
# Analyse each sample (simu)
#
# Look the number of alleles at each makers 
for (locus in 1:NBLOC) {
posit = posit +1
nbal = TOUT[posit]
# Find the allele max
NbMaxAll = max(NbMaxAll,nbal)
# Step to the next maker 
posit = posit + nbal
}
# End
# 
# Second step : Extracting frequencies
#
# Read again
posit = 0
Genot = vector(mode="numeric",length=NbMaxAll)
PkMoy = vector(mode="numeric",length=NbMaxAll)
FreqDis = matrix(nrow=NBLOC,ncol=NbMaxAll)
VarCov = matrix(nrow = NbMaxAll,ncol=NbMaxAll)
# 
# Save the number of alleles at each markers "Nall[locus] = nbal"
#
locut = 0
	for (locus in 1:NBLOC) {
	posit = posit +1
	nbal = TOUT[posit]
	Genot = Genot * 0
   		for (al in 1:nbal) {
   		posit = posit + 1
   		Genot[al] = TOUT[posit]
   		}
# Frequencies
   		nbgenes = sum( Genot[] )
# check sample size
		if( nbgenes > 2 )  {
		locut = locut + 1
		if ( nbgenes < 10 ) {
 		print(c(" Marker ",locus," , low sample size = ",nbgenes), quote="FALSE")
					   }
 		nbgen2  = sum( Genot[]**2 )
# FreqDis(0) = (nbgen2 - nbgenes) / ( nbgenes*(nbgenes-1) )
		FreqDis[locut,1] = nbgen2 - nbgenes
# Initialization
		FreqDis[locut,2:NbMaxAll] = 0
# Browse the pairs of alleles  al1, al2 
  		if( nbal > 1 )  {
   			for (al1 in 1:(nbal-1) ) { for (al2 in (al1+1):nbal ) {
   			delta = al2-al1+1
   			FreqDis[locut,delta] =FreqDis[locut,delta] + Genot[al1]*Genot[al2]
   			}}
            	}
# Standardization
		FreqDis[locut,] = FreqDis[locut,] / ( nbgenes*(nbgenes-1) )
}
					}
# End
if ( locut < NBLOC ) FreqDis[1:locut,] -> FreqDis
# Extracting PkMoy, VarCov
for (delta in 1:NbMaxAll ) { PkMoy[delta] <- mean( FreqDis[,delta] ) }
 VarCov = var( FreqDis )
return( list(L=locut,A=NbMaxAll,P=PkMoy,V=VarCov) )
}
#############################################
datex = .DataExtract(infile,NBLOC)
PkMoy  = datex$P
VarCov = datex$V
NbMaxAll   = datex$A
NELOC      = datex$L
print(" ************************************************************************",quote="FALSE")
print(" ",quote="FALSE")
print(" *** Priors about population sizes ***",quote="FALSE")
print(" ",quote="FALSE")
print(" Preliminary analysis of data to choose Ne priors  ",quote="FALSE")
print(" from global Theta estimates derived from microsatellites data.",quote="FALSE")
print(" ",quote="FALSE")
print(" VarEff calculations make use of normalized scales for size and times,",quote="FALSE")
print(" so that population sizes are expresssed as theta values, Theta = 4Neu,",quote="FALSE")
print(" where Ne is the census population size and u the global mutation rate.",quote="FALSE")
print(" Choosing a mutation rate allows conversion from these normalized scale",quote="FALSE")
print(" to natural values of population size Ne.",quote="FALSE")
#
if(!is.null(MUTAT)){
print (paste(" Mutation rate MUTAT= ",MUTAT,sep=""),quote="FALSE")}else{
MUTAT <- as.numeric(readline(" Give the mutation rate MUTAT = "))} 
#
# Estimation of theta_0, theta_1 et theta_2
#
thetasim <- vector(mode="numeric",length=3)
#
# Theta_1
theta1 = 2 * sum(c(1:(NbMaxAll-1))*PkMoy[2:NbMaxAll])
theta1 = theta1*(theta1+sqrt(theta1**2+1))
thetasim[2]=theta1
# Theta_2
theta2 = 2*sum((c(1:(NbMaxAll-1))**2)*PkMoy[2:NbMaxAll])
thetasim[3]= theta2
# the variance of theta_2 estimates accross markers
#squares = c(1:(NbMaxAll-1))^2
#vartheta2O = t(squares) %*% VarCov[2:NbMaxAll,2:NbMaxAll] %*% squares
# its theoretical expectation for independent markers
#vartheta2T = theta2 * ( 4 * theta2 + 1 ) / 3
#vartheta2T = sqrt(vartheta2T)
#vartheta2O = 2 * sqrt(vartheta2O)
# Theta0
theta0 = PkMoy[1]
theta0 = theta0**2
theta0 = (1/theta0)-1
theta0 = theta0/2
thetasim[1]=theta0
#
# Global values of Theta_0, Theta_1, Theta_2 
#
print(" Theta (4*Ne*u) is estimated using 3 estimators correlated to",quote="FALSE")
print(" Present, Intermediate and Ancestral population sizes.",quote="FALSE")
print(" ",quote="FALSE")
print(" Mean values of estimates Theta_0, Theta_1, Theta_2",quote="FALSE")
print(thetasim,quote="FALSE")
#
print(" Imbalance indices ln(Theta_1/Theta_0) and ln(Theta_2/Theta_0)",quote="FALSE")
ib1 = log(theta1/theta0) 
ib2 = log(theta2/theta0) 
print(c( ib1 , ib2 ),quote="FALSE")
#
#
print(" ",quote="FALSE")
print(" The prior value of population size (Ne) and the assumed mutation rate (MUTAT)",quote="FALSE")
print(" must be in agreement with these estimated theta values"  ,quote="FALSE")
#
MinNe = as.integer(min(thetasim)/(4*MUTAT))
MaxNe = as.integer(max(thetasim)/(4*MUTAT))
print(paste(" i.e. Ne should be of the order of magnitude of ",MinNe," to ",MaxNe) ,quote="FALSE")
#
thetafile = paste(parafile,"Thet0",sep=".")
print(" ",quote="FALSE")
print(" Results of Theta, Imbalances indices, and these Ne values are written to the",quote="FALSE")
print(paste(" ",thetafile," file"),quote="FALSE")
#
#write preliminary results to file .Thet0
write("Theta() preliminary analysis and setting of parameters for VarEff",file=thetafile)
write(paste("VarEff is ready to be run with parameters as in file: ",paste(parafile,".R",sep="")),file=thetafile,append="TRUE")
write(paste("Global results for job ",parafile),file=thetafile,append="TRUE")
write("Global Theta values:",file=thetafile,append="TRUE")
write(paste("   Theta_0 ",thetasim[1]),file=thetafile,append="TRUE")
write(paste("   Theta_1 ",thetasim[2]),file=thetafile,append="TRUE")
write(paste("   Theta_2 ",thetasim[3]),file=thetafile,append="TRUE")
#
write(paste("Corresponding range of Ne values for a mutation rate= ",MUTAT),file=thetafile,append="TRUE")
write(c(MinNe,MaxNe),file=thetafile,append="TRUE")
#
write("Imbalance indices  ",file=thetafile,append="TRUE")
write(paste("   i1 = ln(Theta_1/Theta_0)  ",ib1),file=thetafile,append="TRUE")
write(paste("   i2 = ln(Theta_2/Theta_0)  ",ib2),file=thetafile,append="TRUE")

#########################################################################
#
print(" ",quote="FALSE")
print(" ************************************************************************",quote="FALSE")
print(" ",quote="FALSE")
print(" *** Set parameters for the VarEff algorithm ***",quote="FALSE")
#
# Ask for parameters to run VarEff:
print(" ",quote="FALSE")
print(" *** Maximal distance between alleles (DMAX) *** ",quote="FALSE")
print(" ",quote="FALSE")
print(" DMAX is the maximal distance between alleles in terms of repeat units ",quote="FALSE")
print(" ",quote="FALSE")
print(" Look at the distribution of the f_k frequencies of pairs of alleles",quote="FALSE")
print(" at distance k found in your microsatellite markers data,",quote="FALSE")
print(" and take a large enough maximal allele distance DMAX, ",quote="FALSE")
print(" ",quote="FALSE")
print(" A suggestion is to take DMAX such that frequencies f_k",quote="FALSE")
print(" are under the red line for k > DMAX (i.e. f_k < 0.005 for k > DMAX)",quote="FALSE")
print(" (this choice is generally efficient) ",quote="FALSE")
#
Dk = length(PkMoy)
abscissa = " Distance k between alleles"
ordinates= " Frequencies f_k of pairs of alleles at distance k"
title = paste(" Data file: ", infile)
par(adj=0.5)
plot(c(-0.5,Dk+0.5),c(0,max(PkMoy)),xlab=abscissa,ylab=ordinates,main=title,type='n')
lines(-0.5+c(0:(Dk-1)),PkMoy,type='h')
esfplus = c(PkMoy,0)
lines(-0.5+c(0:Dk),esfplus,type='s')
lines(c(-0.5,Dk-0.5),c(0.005,0.005),col="red")
#
# Choose the DMAXPLUS
#
if(!is.null(DMAXPLUS)){
print (paste(" Maximal distance between the alleles = ",DMAXPLUS,sep=""),quote="FALSE")}else{
DMAXPLUS <- 1 + as.numeric(readline(" Give DMAX = "))}
#
# check that DMAXPLUS is smaller than the largest allele distance
if( DMAXPLUS > NbMaxAll )
{
print(" ERROR !!!",quote="FALSE")
print(c(" DMAX must be smaller than ",NbMaxAll),quote="FALSE")
DMAXPLUS = 1 + as.numeric(readline(" re-enter DMAX : "))
}
# final check !
if( DMAXPLUS > NbMaxAll )
{
print(c("DMAX is reduced to ",(NbMaxAll-1)),quote="FALSE")
DMAXPLUS = NbMaxAll-1
}
#
print(" ",quote="FALSE")
print(" *** Demographic model (JMAX) *** ",quote="FALSE")
print(" Choose the number JMAX of transitions between periods with constant Ne. ",quote="FALSE")
print(" This value is an integer generally between 2 and 6. ",quote="FALSE")
if(!is.null(JMAX)){
print (paste(" Number of transitions =",JMAX,sep=""),quote="FALSE")}else{
JMAX <- as.numeric(readline(" Value of JMAX = "))}
#
print(" ",quote="FALSE")
print(" *** Mutation model (MODEL) *** ",quote="FALSE")
#
print(" ",quote="FALSE")
print(" Choose the mutation model (S, T or G) ",quote="FALSE")
print("   (S = Single Step Model, T = Two Phase Model, G = Geometric Model)",quote="FALSE")
print("   and the value of the C parameter for options T and G,  0 < C < 1",quote="FALSE")
print("   e.g. answer  'S' for the Single Step Model",quote="FALSE")
print("            or ' T   0.15 ' for the Two Phase Model ",quote="FALSE")
print("               with a proportion 15% of two motifs steps",quote="FALSE")
if(!is.null(MODEL)){
print (paste(" Mutation model =",MODEL,sep=""),quote="FALSE")}else{
MODEL = readline(" Mutation model and C coefficient? ")}
#
print(" ",quote="FALSE")
print(" *** Prior of census population size (NBAR) *** ",quote="FALSE")
print("According to global Theta values, NBAR could be ",quote="FALSE")
print(paste("of the order of magnitude of ",MinNe," to ",MaxNe) ,quote="FALSE")
#
if(!is.null(NBAR)){
print (paste(" Prior value of census Ne =",NBAR,sep=""),quote="FALSE")}else{
NBAR = as.numeric(readline(" Give prior of census Ne = "))}
#
print(" ",quote="FALSE")
print(" *** Prior variance of population size in logarithm (VARP1) *** ",quote="FALSE")
print(" a suggested value is VARP1=3 that allows for searches with 20- to 40-fold",quote="FALSE")
print(" relative variations of effective size.",quote="FALSE")
if(!is.null(VARP1)){
print (paste(" Prior variance of log(Ne) =",VARP1,sep=""),quote="FALSE")}else{
VARP1 = as.numeric(readline(" Give the prior variance of log(Ne) = "))}
#
print(" ",quote="FALSE")
print(" *** Prior correlation between successive population sizes (RHOCORN) *** ",quote="FALSE")
print(" Generally RHOCORN can be set to 0, meaning that there is no correlation.",quote="FALSE")
if(!is.null(RHOCORN)){
print (paste(" Prior correlation between successive population sizes =",RHOCORN,sep=""),quote="FALSE")}else{
RHOCORN = as.numeric(readline(" Give the prior correlation between successive population sizes = "))}
#
print(" ",quote="FALSE")
print(" *** Priors about time and time intervals (GBAR) (number of generations)***",quote="FALSE")
print(" ",quote="FALSE")
print(" *** Number of generations since the assumed origin of the population.",quote="FALSE")
if(!is.null(GBAR)){
print (paste(" Prior time since the Origin (number of generations) =",GBAR,sep=""),quote="FALSE")}else{
GBAR =as.numeric(readline(" Give the prior time since the Origin = "))}
# 
print(" ",quote="FALSE")
print(" *** Prior variance of the time since the origin (VARP2) *** ",quote="FALSE")
print(" As for VARP1, a value of 3 may be suggested.",quote="FALSE")
if(!is.null(VARP2)){
print (paste(" Prior Variance of log(time interval) =",VARP2,sep=""),quote="FALSE")}else{
VARP2 = as.numeric(readline(" Give the Prior Variance of log(time interval) = "))}
#
print(" ",quote="FALSE")
#
# MCMC
print(" *** Smoothing the Covariance matrix (Diagonale parameter) ***",quote="FALSE")
print(" ",quote="FALSE")
print("    Calculation of an approximate likelihood assumes the theoretical",quote="FALSE") 
print("    variance-covariance matrix V of observations is known.",quote="FALSE") 
print("    It is in fact replaced by a combination of the sample estimate Vs",quote="FALSE")
print("    and of a diagonal matrix Vd made of theoretical variances:",quote="FALSE")
print("    V = (1-D) * Vs + D * Vd",quote="FALSE")
print("     D  must be > 0 and less than 1",quote="FALSE")
print("     D = 1  means that only theoretical variances are considered",quote="FALSE") 
print("            and correlations are ignored ",quote="FALSE")
print("     D = 0  means that only observed data are considered, a choice ",quote="FALSE")
print("            that may raise numerical instability",quote="FALSE")
print("     For more detail read the help and article",quote="FALSE")
print("     A medium choice, D = 0.5 , is generally robust",quote="FALSE")
#
if(!is.null(Diagonale)){
print (paste(" Covariance matrix = ",Diagonale,sep=""),quote="FALSE")}else{
Diagonale= as.numeric(readline(" Enter D value = "))}
#
Diagonale = min(Diagonale,1)
if( Diagonale <= 0) 
{
print(" D must be > 0, it is reset to 0.1",quote="FALSE")
Diagonale = 0.1
}
#
print(" *** Length of the MCMC chain (look at the metrop specifications) ***",quote="FALSE")
print(" ",quote="FALSE")
print(" Suggested initial values: ",quote="FALSE")
print("    nbatch = 10000, blen=10, nspac=10 , Burnin=10000, AccRate=0.25",quote="FALSE")
print(" (look at the documentation of the mcmc package, Geyer 1999)",quote="FALSE")
print(" Enter the five values you wish to use",quote="FALSE")
if(!is.null(NumberBatch)){
print (paste(" nbatch = ",NumberBatch,sep=""),quote="FALSE")}else{
NumberBatch = as.numeric(readline(" nbatch = "))}
if(!is.null(LengthBatch)){
print (paste(" blen = ",LengthBatch,sep=""),quote="FALSE")}else{
LengthBatch = as.numeric(readline(" blen = "))}
if(!is.null(SpaceBatch)){
print (paste(" nspac = ",SpaceBatch,sep=""),quote="FALSE")}else{
SpaceBatch = as.numeric(readline(" nspac = "))}
if(!is.null(Burnin)){
print (paste(" Burnin length = ",Burnin,sep=""),quote="FALSE")}else{
Burnin = as.numeric(readline(" Burnin length = "))}
if(!is.null(AccRate)){
print (paste(" Acceptation rate = ",AccRate,sep=""),quote="FALSE")}else{
AccRate= as.numeric(readline(" Acceptation rate = "))}
#
#  save the VarEff file 
namecom = paste(parafile,".R",sep="")
print (paste(" Name of the parameter set = ",parafile,sep=""),quote="FALSE")
print(paste(" Enter the command: source('",namecom,"') to launch the VarEff function",sep=""),quote="FALSE")
.parDiagwrite(parafile,infile,NBLOC,JMAX,MODEL,MUTAT,NBAR,VARP1,RHOCORN,GBAR,VARP2,DMAXPLUS,Diagonale,NumberBatch,LengthBatch,SpaceBatch,Burnin,AccRate)
#############################################
return()
}
###############################################################################################
#  Main function
#
VarEff <- function(parafile=NULL,
                         infile=NULL,
                         NBLOC=NULL,
                         JMAX=NULL,
                         MODEL=NULL,
                         MUTAT=NULL,
                         NBAR=NULL,
                         VARP1=NULL,
                         RHOCORN=NULL,
                         GBAR=NULL,
                         VARP2=NULL,
                         DMAXPLUS=NULL,
                         Diagonale=NULL,
                         NumberBatch=NULL,
                         LengthBatch=NULL,
                         SpaceBatch=NULL,
                         Burnin=NULL,
                         AccRate=NULL)
{
# 
print(" VarEff package (Version 2) - main function VarEff()",quote="FALSE")
print("       Estimation of effective sizes Ne(T) in the past",quote="FALSE")
print("       from microsatellite markers data",quote="FALSE")
print("       Note that this R code makes use of the mcmc library (Geyer 1999)",quote="FALSE")
print(" ",quote="FALSE")
#
#############################################
#
# If not given, give a name for the job
if(!is.null(parafile)){
print(paste(" Name of the job = ",parafile,sep=""),quote="FALSE") }else
{parafile = readline(" Give a name for this job = ")}
#
# Parameters for the model, given by the user
# 
print(" *** Data ***",quote="FALSE")
if(!is.null(infile)){
print (paste(" Name of the data file= ",infile,sep=""),quote="FALSE")}else{
infile = readline(" Name of the data file= ")}
#
if(!is.null(NBLOC)){
print (paste(" Number of microsatellite markers= ",NBLOC,sep=""),quote="FALSE")}else{ 
NBLOC = as.numeric(readline(" Number of microsatellite markers= "))}
#
print(" *** Demographic model ***",quote="FALSE")
print(" Choose the number of transitions between periods with constant Ne ",quote="FALSE")
if(!is.null(JMAX)){
print (paste(" Value of JMAX = ",JMAX,sep=""),quote="FALSE")}else{
JMAX <- as.numeric(readline(" Value of JMAX = "))}
#
print(" *** Mutation model ***",quote="FALSE")
print(" Fix the mutation model (S, T or G) ",quote="FALSE")
print("   (S = Single Step Model, T = Two Phase Model, G = Geometric Model)",quote="FALSE")
print("   and the prior value of the C parameter for models T or G,  0 < C < 1",quote="FALSE")
print("   e.g. answer  'S' for the Single Step Model",quote="FALSE")
print("            or ' T   0.15 ' for the Two Phase Model ",quote="FALSE")
print("               with a proportion 15% of two motifs steps",quote="FALSE")
if(!is.null(MODEL)){
print (paste(" MODEL = ",MODEL),quote="FALSE")}else{
MODEL = readline(" Mutation model and C coefficient? ")
}
CoGeo = .MutMod(MODEL)
#
print(" Give the mutation rate MUTAT",quote="FALSE")
if(!is.null(MUTAT)) {
print (paste(" MUTAT = ",MUTAT),quote="FALSE")}else{
MUTAT <- as.numeric(readline(" MUTAT = "))}
#
print(" *** Priors about population sizes ***",quote="FALSE")
print(" ",quote="FALSE")
#
print(" Prior expected value of Effective Population Size Ne:",quote="FALSE")
if(!is.null(NBAR)){
print (paste(" Prior Ne= ",NBAR,sep=""),quote="FALSE")}else{
NBAR = as.numeric(readline(" Prior Ne = "))}
#
if(!is.null(VARP1)){
print (paste(" Variance of log(Ne)= ",VARP1,sep=""),quote="FALSE")}else{
VARP1 = as.numeric(readline(" Variance of log(Ne)= "))}
#
print(" Prior correlation between successive population sizes:",quote="FALSE")
#
if(!is.null(RHOCORN)){
print (paste(" correlation= ",RHOCORN,sep=""),quote="FALSE")}else{
RHOCORN = as.numeric(readline(" correlation= "))}
#
print(" *** Priors about time and time intervals ***",quote="FALSE")
print(" ",quote="FALSE")
#
print(" Prior time since the Origin (number of generations))",quote="FALSE")
if(!is.null(GBAR)){
print (paste(" Origin time (generations)= ",GBAR,sep=""),quote="FALSE")}else{
GBAR =as.numeric(readline(" Origin time (generations) = "))}
# 
print(" Prior about time intervals:",quote="FALSE")
#
if(!is.null(VARP2)){
print (paste(" Variance of log(time interval)= ",VARP2,sep=""),quote="FALSE")}else{
VARP2 = as.numeric(readline(" Variance of log(time interval)= "))}
#
# MCMC
#
print(" *** Smoothing the Covariance matrix ***",quote="FALSE")
print(" ",quote="FALSE")
print("    Calculation of an approximate likelihood assumes the theoretical",quote="FALSE") 
print("    variance-covariance matrix V of observations is known.",quote="FALSE") 
print("    It is in fact replaced by a combination of the sample estimate Vs",quote="FALSE")
print("    and of the diagonal matrix Vd made of theoretical variances:",quote="FALSE")
print("    V = (1-D) * Vs + D * Vd",quote="FALSE")
print("     D  must be > 0 and less than 1",quote="FALSE")
print("     D = 1  means that only theoretical variances are considered",quote="FALSE") 
print("            and correlations are ignored ",quote="FALSE")
print("     D = 0  means that only observed data are considered, a choice ",quote="FALSE")
print("            that may raise numerical instability",quote="FALSE")
print("     For more detail read the help and article",quote="FALSE")
print("     A medium choice, D = 0.5 , is generally robust",quote="FALSE")
#
if(!is.null(Diagonale)){
print (paste(" D = ",Diagonale,sep=""),quote="FALSE")}else{
Diagonale= as.numeric(readline(" D = "))}
#
Diagonale = min(Diagonale,1)
if( Diagonale <= 0) 
{
print(" Diagonale must be > 0, it is reset to 0.1",quote="FALSE")
Diagonale = 0.1
}
#
print(" *** Length of the MCMC chain (look at the metrop specifications) ***",quote="FALSE")
print(" Suggested initial values: ",quote="FALSE")
print("   nbatch = 10000, blen=10, nspac=10 , Burnin=10000, AccRate=0.25",quote="FALSE")
print(" (look at the documentation of the mcmc package, Geyer 1999)",quote="FALSE")
print(" Enter the five values you wish to use",quote="FALSE")
if(!is.null(NumberBatch)){
print (paste(" nbatch = ",NumberBatch,sep=""),quote="FALSE")}else{
NumberBatch = as.numeric(readline(" nbatch = "))}
if(!is.null(LengthBatch)){
print (paste(" blen = ",LengthBatch,sep=""),quote="FALSE")}else{
LengthBatch = as.numeric(readline(" blen = "))}
if(!is.null(SpaceBatch)){
print (paste(" nspac = ",SpaceBatch,sep=""),quote="FALSE")}else{
SpaceBatch = as.numeric(readline(" nspac = "))}
if(!is.null(Burnin)){
print (paste(" Burnin length = ",Burnin,sep=""),quote="FALSE")}else{
Burnin = as.numeric(readline(" Burnin length = "))}
if(!is.null(AccRate)){
print (paste(" Acceptation rate = ",AccRate,sep=""),quote="FALSE")}else{
AccRate= as.numeric(readline(" Acceptation rate = "))}
#
#############################################
# Fixing constants of the prior distributions
#
# checking JMAX, a parameter that must be > 0
# if JMAX is entered as 0 (the model with a single constant population
# size, it is changed to  JMAX = 1, with RHOCORN = 0.999 so as to
# simulate a single population size
if( JMAX == 0 )
{
JMAX = 1
RHOCORN = 0.999
}
   JMAX1 <- JMAX+1
   JMAX2 <- JMAX+2
   COEF2 <- - 1/(2*VARP2)
   COEF1 <- - 1/(2*VARP1*(1-RHOCORN^2))
#
# add priors for extensions of the mutation model
   NBPARAM <- 2*JMAX+2
#
# in the present version of the program, the additional parameter C is 
# assumed fixed throughout the calculations
      Prod3 = 0
      COEF3 = 0 
      VARP3 = 0
#      if( !(CoGeo == 0 ) )
#   {
#      Prod3 = 0.5
#      VARP3 = 0.69
#      COEF3 = - 1/( 2* VARP3)
#    }
#
#############################################
# Initialisation of the set of parameters to be searched for
#
   parametre = rep(0,NBPARAM)
   THETATAUZERO <- vector(mode="numeric",length=NBPARAM)
   the <- 4*MUTAT*NBAR 
   gba <- GBAR/(2*NBAR*JMAX)
   THETATAUZERO[1:JMAX1] <- the
   THETATAUZERO[JMAX2:(NBPARAM-1)] <- gba
#
#  Priors mean for the model of mutation (fixed in the present version)
#
   THETATAUZERO[NBPARAM] = CoGeo
#
priors = list(TZ=THETATAUZERO,C1=COEF1,C2=COEF2,C3=COEF3,RO=RHOCORN)
#
##############################################
# Definition of the proposal distribution (matrix scalecal)
#
   SCALE11 <- t(chol(.vdmdiretendue(RHOCORN, JMAX)))
  scalecal <- SCALE11*.prodEE(JMAX,0.5,0.5,Prod3)
  paraminit <- parametre
#
#############################################
# Reading the data from infile
#
datex = .DataExtract(infile,NBLOC)
PkMoy  = datex$P
VarCov = datex$V
NbMaxAll   = datex$A
NELOC      = datex$L
#
print(c(" Effective Number of markers = ",NELOC)  ,quote="FALSE" )
#
# Showing the distribution of P_k from data to choose DMAXPLUS
#
print(" Plot of the (mean) distribution of distances between alleles",quote="FALSE")
###
abscissa = " Distance k between alleles"
ordinates= " Frequencies f_k of pairs of alleles at distance k"
title = paste(" Data file: ", infile)
par(adj=0.5)
plot(c(-0.5,NbMaxAll   +0.5),c(0,max(PkMoy)),xlab=abscissa,ylab=ordinates,main=title,type='n')
lines(-0.5+c(0:(NbMaxAll   -1)),PkMoy,type='h')
esfplus = c(PkMoy,0)
lines(-0.5+c(0:NbMaxAll   ),esfplus,type='s')
#
lines(c(-0.5,NbMaxAll   -0.5),c(0.005,0.005),col="red")
#
# save the plot of f_k frequencies to file
#dev.copy2pdf(file=paste(infile,"_Pk.pdf",sep=""))
#
# Choose DMAXPLUS
#
print(" Choose the range 0:DMAX of allele distances to be analysed",quote="FALSE")
print(" e.g. so that f_k < 0.005 for k > DMAX (under the red line)",quote="FALSE")
if(!is.null(DMAXPLUS)){
print (paste(" DMAX = ",(DMAXPLUS-1),sep=""),quote="FALSE")}else{
DMAXPLUS = 1 + as.numeric(readline(" Enter DMAX = "))}
#
# check that DMAXPLUS is smaller than the largest allele distance
if(DMAXPLUS > NbMaxAll)
{
print(c(" DMAX is reduced to ",(NbMaxAll-1)),quote="FALSE")
DMAXPLUS = NbMaxAll
}
#############################################
#  Save this set of parameters to file 
#
.parDiagwrite(parafile,infile,NBLOC,JMAX,MODEL,MUTAT,NBAR,VARP1,RHOCORN,GBAR,VARP2,DMAXPLUS,Diagonale,NumberBatch,LengthBatch,SpaceBatch,Burnin,AccRate)
#
#############################################
# Mathematical resolutions
#############################################
# Markov Chains with a priori and the parameters 
# NbMaxAll, PkMoy, VarCov, DMAXPLUS, 
#
# Global estimates of a single theta value
#
# theta_0, theta_1 et theta_2 are put in thetasim[1:3]
#
thetasim <- vector(mode="numeric",length=7)
#
# Estimation of theta_0, theta_1 et theta_2
#
# Theta_1
theta1 = 2 * sum(c(1:(NbMaxAll-1))*PkMoy[2:NbMaxAll])
theta1 = theta1*(theta1+sqrt(theta1**2+1))
thetasim[2]=theta1
# Theta_2
theta2 = 2*sum((c(1:(NbMaxAll-1))**2)*PkMoy[2:NbMaxAll])
thetasim[3]= theta2
# Theta0
theta0 = PkMoy[1]
theta0 = theta0**2
theta0 = ( 1/theta0 )-1
theta0 = theta0/2
thetasim[1]=theta0
#
# Outputs
#
# Saving the results on files (start)
# Global theta estimates
print(" Mean values of estimates Theta_0, Theta_1, Theta_2",quote="FALSE")
print(thetasim[1:3],quote="FALSE")
print(" Imbalance indices ln(Theta_1/Theta_0) and ln(Theta_2/Theta_0)",quote="FALSE")
ib1 = log(theta1/theta0) 
ib2 = log(theta2/theta0) 
print(c( ib1 , ib2 ),quote="FALSE")
#
thetafile = paste(parafile,"Theta",sep=".")
write(paste("VarEff global results for job ",parafile),file=thetafile)
write(paste("VarEff was run with parameters as in file: ",paste(parafile,".R",sep="")),file=thetafile,append="TRUE")
write("Global Theta values:",file=thetafile,append="TRUE")
write(paste("   Theta_0 ",thetasim[1]),file=thetafile,append="TRUE")
write(paste("   Theta_1 ",thetasim[2]),file=thetafile,append="TRUE")
write(paste("   Theta_2 ",thetasim[3]),file=thetafile,append="TRUE")
#
write("Imbalance indices  ",file=thetafile,append="TRUE")
write(paste("   i1 = ln(Theta_1/Theta_0)  ",ib1),file=thetafile,append="TRUE")
write(paste("   i2 = ln(Theta_2/Theta_0)  ",ib2),file=thetafile,append="TRUE")
#
# Likelihood model
#
# "Diagonale" is entered as a parameter
#
FREQEXPE <- PkMoy[1:DMAXPLUS]
VCDMX = VarCov[1:DMAXPLUS,1:DMAXPLUS]
# The approximate theoretical Diagonal 
diagt = .diagtheo(theta1,DMAXPLUS)
VCDMD = Diagonale*diag(diagt)+ (1-Diagonale)*VCDMX 
VARCOVINV = solve(VCDMD)
#
# Get the constant term of quadratic deviations
XLKZ = 0
XLKZ = t(FREQEXPE) %*% ( VARCOVINV %*% FREQEXPE )
#
#
# V2 / expectation
# ExpQuad =  sum( VCDMX * VARCOVINV )
# print(paste(" V2 test, Exp Quad =  ",ExpQuad))
# test V2 / expectation
# exp of variance under normality
# ProdXV = VCDMX %*% VARCOVINV 
# sdquad = sqrt( 2 * sum( ProdXV * ProdXV ) )
# print(paste(" V2 test, sd Quad =  ",sdquad ))
#
# Get the constant term of the prior log-probability
PZZERO = 0
# If(VARP3 > 0) PZZERO = log(VARP3)
PZZERO = PZZERO + NBPARAM * log(2*pi) + JMAX*log(VARP2)
PZZERO = PZZERO + (JMAX+1)*log(VARP1) + JMAX*log(1-RHOCORN^2)
PZZERO = - PZZERO / 2
#
# Keep observed values in the list  "observed"
#
observed = list(DM=DMAXPLUS,NL=NELOC,FR=FREQEXPE,IV=VARCOVINV)
# 
#############################################
# Adjust "scalecal" to get the chosen acceptation rate of MCMC (AccRate parameter)
# REFACC=0.25
ResuRefa = .Refa2(.Laposter,paraminit,JMAX,observed,priors,AccRate,scalecal)
scalecal = ResuRefa$SC
# 
#############################################
# Burn-in
out <- metrop(ResuRefa$OO,nbatch=Burnin,blen=1,nspac=1,scale=scalecal,,,,,JMAX,observed,priors)
#
#############################################
# Chain 
out <-  metrop(out,nbatch=NumberBatch,blen=LengthBatch,nspac=SpaceBatch,,,,,,JMAX,observed,priors)
#
#############################################
# Recover Posterior probabilities and likelihoods of the simulated states
#
distlp = .distriL(out$batch,JMAX,observed,priors)
distLike = distlp$L
distPost = distlp$P
#
# Quadratic deviation from expectation (f_obs-f_exp)' Varcovinv (f_obs-f_exp)
Quaddev = - 2 *  distLike + NELOC * XLKZ
#
# Record Quadev and distPost stats in thetasim
thetasim[4] = mean(Quaddev)
thetasim[5] = sd(Quaddev)
thetasim[6] = mean(distPost-distLike) + PZZERO
thetasim[7] = sd(distPost-distLike)
#
# Recover parameters to their natural values (reduced scales theta and G*u times))
tt = .ExtracNatural(out$batch,JMAX,priors$TZ,MUTAT)
# 
#############################################
# Outputs
#
# Saving the results to file thetafile 
#
print(c(" Realized acceptation rate: ",out$accept),quote="FALSE")
print(" Mean and standard deviation of Quadratic deviations from data",quote="FALSE")
print(thetasim[4:5],quote="FALSE")
print(" Mean and standard deviation of log prior probabilities",quote="FALSE")
print(thetasim[6:7],quote="FALSE")
#
write(paste("Realized acceptation rate: ",out$accept),file=thetafile,append="TRUE")
# V2 quad-add
# write(paste("Quadratic deviations, expected mean            ",ExpQuad),file=thetafile,append="TRUE")
# write(paste("Quadratic deviations, normal sd                ",sdquad),file=thetafile,append="TRUE")
#
write(paste("Mean of quadratic deviations (observed):  ",thetasim[4]),file=thetafile,append="TRUE")
write(paste("  sd of quadratic deviations (observed):  ",thetasim[5]),file=thetafile,append="TRUE")
write(paste("Mean of log prior probabilities:          ",thetasim[6]),file=thetafile,append="TRUE")
write(paste("  sd of log prior probabilities:          ",thetasim[7]),file=thetafile,append="TRUE")
#
# The detailed series of simulations
# 
# File .Batch    the full set of simulated values 
#           column 1 = Number of the simulation
#           column 2 = Quaddev
#           column 3 = Prior proba of parameters
#           columns 3+1 to 3+JMAX+1: theta_j
#           columns 4+JMAX+1 to 4+JMAX+JMAX: times t_j in generation*mutation units
#           column  5 + 2*JMAX: the current C value (extended mutation models)
#                      for Geometric model, C is encoded with positive values
#                      for the Two Phase Model, C is encoded as its opposite
#
fullout = matrix(c(1:NumberBatch,Quaddev,distPost-distLike+PZZERO,tt),nrow=NumberBatch)
write(t(fullout),ncolumns=(NBPARAM+3),file=paste(parafile,"Batch",sep="."))
return()
}
##############################################################################
############################################################################## 
# Visualization of the results of VarEff
# 
#    Note that if the parameter MUTAT is set to 0 in these functions,
#    results are shown using reduced scales (theta's, and G*u units).
#    Any intermediate result is saved under this reduced format.
#
##############################################################################
# Effective size distribution functions:
#    natural values (Ne or theta scales):              Function  NatSizeDist(BATCHfile,TMAX,NBT,MUTAT)
# or decimal logarithms (log_10(Ne) or log_10(theta)): Function  LogSizeDist(BATCHfile,TMAX,NBT,MUTAT)
#
# Extracts Ne distribution at a number of past times
# Input:    the name of the parafile.Batch file produced by VarEff
#           the interval of observation of past times TMAX 
#           the number of observed time intervals NBT (hence NBT+1 times from 0 to TMAX)
#           the MUTAT mutation rate (if set to 0, TMAX must be given in reduced time,
#                                    and results are given in reduced scales)
#
# Results:  Global stats (mean of posterior distributions of (log)Ne at past times),
#           given as a table with (NBT+1) lines and 8 columns containing each:
#           time of observation, arithmetic mean, harmonic mean, mode, 
#           median, 5% and 95% quantiles, and standard deviation(added in Version 1.1 and 1.2)
#           The table is written to file parafile.Nstat, and the results plotted.
#              
#           In addition the densities of the posterior distribution of (log)Ne at the analysed times
#           are written to file parafile.Ndist (to be visualized with the plotNdistrib() function, below).
#
#      technical comment on the code:
#      The values extrait$tK, -$moy, -$eca, -$Q and -$stade are in units theta = 4 * N * MUTAT 
#      The values  extrait$tech  are in units Generation * MUTAT
#
#   parametre ",NatSizefile=NULL"  SUPPRIME
#
NatSizeDist <- function(NameBATCH=NULL,MUTAT=NULL,TMAX=NULL,NBT=NULL)
{
# get parameters
print(" VarEff package - NatSizeDist() function",quote="FALSE")
print("          View past effective sizes Ne(T)",quote="FALSE")
print("          and save posterior distributions",quote="FALSE")
if(!is.null(NameBATCH)){
print (paste(" Name of the batch table= ",NameBATCH),quote="FALSE")}else{
NameBATCH = readline(" Name of the batch file= ")}
#
if(is.null(MUTAT))
{
print(" Mutation rate: Enter 0 to use reduced scales Theta's and G*u",quote="FALSE")
MUTAT = as.numeric(readline(" mutation rate = "))
}
#
if(!is.null(TMAX)){
print (paste(" Length of the period = ",TMAX),quote="FALSE")}else{
print(" Enter the length of the period to be analysed",quote="FALSE")
if(MUTAT > 0) print(" give time in number of generations ",quote="FALSE")
if(MUTAT == 0)print(" give time in the reduced time scale G*u ",quote="FALSE")
TMAX = as.numeric(readline(" Length of the period = "))}
#
if(!is.null(NBT)){
print (paste(" Number of time intervals= ",NBT),quote="FALSE")}else{
NBT <- as.numeric(readline(" Number of time intervals= "))}
#
#
if(!(MUTAT == 0))
{
print (paste(" Assumed mutation rate= ",MUTAT),quote="FALSE")
TMAX = TMAX * MUTAT 
}
else
{
print(" Results are expressed in reduced scales Theta's and G*u",quote="FALSE") 
}
#
.LSD(NameBATCH,TMAX,NBT,MUTAT,FALSE)
return()
}
##############################################################################
# Function LogSizeDist(BATCHfile,MUTAT,TMAX,NBT)
# 
# same as SizeDist() for log-sizes
#
LogSizeDist <- function(NameBATCH=NULL,MUTAT=NULL,TMAX=NULL,NBT=NULL)
{
# get parameters
print(" VarEff package - LogSizeDist() function",quote="FALSE")
print("          View past effective sizes log(Ne(T))",quote="FALSE")
print("          and save posterior distributions",quote="FALSE")
if(!is.null(NameBATCH)){
print (paste(" Name of the batch table= ",NameBATCH),quote="FALSE")}else{
NameBATCH = readline(" Name of the batch file= ")}
#
if(is.null(MUTAT))
{
print(" Mutation rate: Enter 0 to use reduced scales Theta's and G*u",quote="FALSE")
MUTAT = as.numeric(readline(" mutation rate = "))
}
if(!is.null(TMAX)){
print (paste(" Length of the period = ",TMAX),quote="FALSE")}else{
print(" Enter the length of the period to be analysed",quote="FALSE")
if(MUTAT > 0) print(" give time in number of generations ",quote="FALSE")
if(MUTAT == 0)print(" give time in the reduced time scale G*u ",quote="FALSE")
TMAX = as.numeric(readline(" Length of the period = "))}
if(!is.null(NBT)){
print (paste(" Number of time intervals= ",NBT),quote="FALSE")}else{
NBT <- as.numeric(readline(" Number of time intervals= "))}
if(!(MUTAT == 0))
{
print (paste(" Assumed mutation rate= ",MUTAT),quote="FALSE")
TMAX = TMAX * MUTAT 
}
else
{
print(" Results are expressed in reduced scales Theta's and G*u",quote="FALSE") 
}
.LSD(NameBATCH,TMAX,NBT,MUTAT,TRUE)
return()
}
#
##############################################################################
# Function NTdist(BATCHfile,TMAX,MUTAT)
# Graphical summary of the 2D joint distribution of N and of T
# Input:    the name of the parafile.Batch file produced by VarEff
#           the interval of observation of past times TMAX 
#           the MUTAT mutation rate (if set to 0, TMAX must be given in reduced time,
#                                    and results are given in reduced scales)
# Results:  the 2D joint distribution of T and log_10( N(T) )
#                  plot using scales as defined by parameter MUTAT 
#                        (reduced scales if MUTAT set to 0)
#              and file parafile.2D using the reduced scales
#
NTdist <- function(NameBATCH=NULL,MUTAT=NULL,TMAX=NULL)
{
print(" VarEff package -  NTdist() function",quote="FALSE")
print("          View joint distribution of T and Ne(T)",quote="FALSE")
print("          plot and save this joint posterior distributions",quote="FALSE")
# get parameters
if(!is.null(NameBATCH)){
print (paste(" Name of the batch file= ",NameBATCH),quote="FALSE")}else{
NameBATCH = readline(" Name of the batch file= ")}
#
if(is.null(MUTAT))
{
print(" Mutation rate: Enter 0 to use reduced scales Theta's and G*u",quote="FALSE")
MUTAT = as.numeric(readline(" mutation rate = "))
}
if(!is.null(TMAX)){
print (paste(" Length of the period = ",TMAX),quote="FALSE")
}else{
print(" Enter the length of the period to be analysed",quote="FALSE")
if(MUTAT > 0) print(" give time in number of generations ",quote="FALSE")
if(MUTAT == 0)print(" give time in the reduced time scale G*u ",quote="FALSE")
TMAX = as.numeric(readline(" Length of the period = "))
}
if(MUTAT == 0) MUTAT=NULL
if(!is.null(MUTAT))
{
print (paste(" Assumed mutation rate= ",MUTAT),quote="FALSE")
TMAX = TMAX * MUTAT 
}
else
{
print(" Results are expressed in reduced scales Theta's and G*u",quote="FALSE") 
}
parafile = substring(NameBATCH,1,nchar(NameBATCH)-6)
BATCH = read.table(NameBATCH)
Lenfull = length(BATCH[1,])
NumberBatch = length(BATCH[,1])
JMAX = (Lenfull-3)/2 - 1 
tt = BATCH[,4:Lenfull]
#
Ntime = 100
Nsize = 100
#
map = .LogMap(tt,TMAX,Ntime, Nsize,JMAX)
#
MinLN = map$sm 
MaxLN = map$SM 
Cover = 100*sum(map$M)/(NumberBatch*Ntime)
# save to file, adding 1st line for times, 1st column for log_10(theta)'s
abscissa = TMAX*c(1:Ntime)/Ntime
ordinates = (MinLN+c(1:Nsize)*(MaxLN-MinLN)/Nsize)/log(10)
mapww = matrix(0,ncol = (Nsize+1),nrow=(Ntime+1) )
mapww[1,1] = Cover
mapww[2:(Nsize+1),1] = abscissa
mapww[1,2:(Ntime+1)] = ordinates
mapww[2:(Nsize+1),2:(Ntime+1)] = map$M
 write(t(mapww),ncolumns=(Nsize+1),file=paste(parafile,"2D",sep=".")) 
# 
MinLN = ( map$sm )/log(10)
MaxLN = ( map$SM )/log(10)
#
if(is.null(MUTAT))
{
print( " Plotting the 2D ( T,log_10(Theta) ) distribution", quote="FALSE")
print( c(" Time range: from 0 to ",TMAX," (G*u units)"),quote="FALSE")
print( " Minimum and Maximum values of log_10(theta) in the plot:",quote="FALSE")
print( c("       Minimum = ",MinLN),quote="FALSE")
print( c("       Maximum = ",MaxLN),quote="FALSE")
}
else
{
print( " Plotting the 2D ( g ,log_10( Ne(g) ) ) distribution", quote="FALSE")
print( c(" Time range: from 0 to ",TMAX/MUTAT," (generations)"),quote="FALSE")
delta = log(4*MUTAT)/log(10)
print( " Minimum and Maximum values of log_10( Ne ) in the plot:",quote="FALSE")
print( c("       Minimum = ",MinLN-delta),quote="FALSE")
print( c("       Maximum = ",MaxLN-delta),quote="FALSE")
}
print( c("       Coverage  = ",Cover, " %"),quote="FALSE")
#
# plot
titre = paste("N-T joint distribution (coverage = ",Cover," % )")
if(is.null(MUTAT))
{
legord=" log_10( Theta(T) )"
legabs= " Time T in the past (unit = G*u)"
TMT = TMAX / Ntime
MinOrdo = MinLN
} 
else
{
legord=" log_10( N(T) )"
legabs= " Time T in the past (generations)"
TMT = TMAX/(Ntime*MUTAT)
MinOrdo = MinLN - delta
}
par(adj=0.5)
image((TMT*c(1:Ntime)),MinOrdo+ c(1:Nsize)*(MaxLN-MinLN)/Nsize,(-map$M),xlab=legabs,ylab=legord)
par(adj=0)
title(main=titre,sub=parafile,cex.main=1,cex.sub=1)
#
# save the graph to pdf file
#if(!is.null(parafile)) { dev.copy2pdf(file=paste(parafile,"_2D.pdf",sep="") ) }
#else {dev.copy2pdf(file=paste(parafile,"_2D.pdf",sep="")}
#return() 
# save the graph to pdf file
#dev.copy2pdf(file=paste(parafile,"_2D.pdf",sep="") )
return()
}
#
##############################################################################
# Function plotNdistrib(NDISTfile)
# Plots posterior distributions of N(T) at given times
# Input:    the name of a parafile.Ndist file produced by the function functions 
#           NatSizeDist() and LogSizeDist()
#           (the choice of scales is made in function SizeDist)
# Results:  plots of the posterior distributions of population size at different
#           past times. Choosing these times and  the range of population sizes
#           to plot is under user's control.
#
plotNdistrib <- function(infile=NULL, nbcases=NULL)
{
print(" VarEff package -  plotNdistrib() function ",quote="FALSE")
print("      Plot posterior distributions of N(T) or log(N(T)",quote="FALSE")
print("      from a .Ndist or .Ldist file previously generated by)",quote="FALSE")
print("      the NatSizeDist() or the LogSizeDist() function",quote="FALSE")
if(!is.null(infile)){
print (paste(" Name of the .Ndist file or .Ldist file= ",infile),quote="FALSE")}else{
infile = readline(" Name of the 'parafile.Ndist' file or 'parafile.Ldist' = ") }
#
parafile = substring(infile,1,nchar(infile)-6)
#
dat=read.table(infile)
# check number of times and show intermediate times
Nbtimes = length(dat[,1])
# check the format of .Ndist file
# 1025 corresponds to log option
XAvail= FALSE
if( length(dat[1,]) == 1025 ) XAvail=TRUE
#
print("Instant       past time ",quote="FALSE")
for (time in 1:Nbtimes) print(paste(time,"                 ",dat[time,1]),quote="FALSE")
# choose the number of instants you wish to plot the distribution
#
#nbcases = 1
#
#while (nbcases > 0) {
#
if(!is.null(nbcases)){
print("Choose the number of times (suggestion: <=5) you wish to plot the distribution",quote="FALSE")}else{
nbcases = as.integer(readline("  number of times (stop on 0): "))
}
if (nbcases > 0) {
if(nbcases > 11) {
return( print( paste(nbcases," is too large a number. Please retry !"),quote="FALSE") )
}
if(nbcases == 0) {
# save the last graph to pdf file
#if(XAvail) { dev.copy2pdf(file=paste(parafile,"_Ldist.pdf",sep="") ) }
#else { dev.copy2pdf(file=paste(parafile,"_Ndist.pdf",sep="") )  }
#return()
                 }
cases = c(1:nbcases)
for (ti in 1:nbcases)
{
cases[ti] = readline(paste(" case # ",ti,", instant number = : "))
}
# define colours
colours = c("black","blue","red","green","grey","purple","orange","pink","brown","yellow","turquoise")
time = cases[1]
if(XAvail) 
{
x = as.numeric(dat[time,2:513])
y = as.numeric(dat[time,514:1025])
}
else
{  
x=as.numeric(dat[time,2])
x= x*c(0:511)
y=as.numeric(dat[time,3:514])
}
z=as.integer( dat[time,1] )
#
if( nbcases == 1) {
legabs =  " Ne  values at time  -"
if(XAvail) legabs =  " log_10(Ne)  values at time  -"
par(adj=0.5)
plot(x,y,xlab=paste(legabs,z,"   "), ylab="Density",type="l")
par(adj=0)
title(sub= parafile)
}
else {
# with several distributions, check limits
if(XAvail)
{
xmax = max( dat[cases,513] )
xmin = min( dat[cases,2] )
}
else
{
xmax = 511 * max( dat[cases,2] )
xmin = 0
}
# choice of the range of (log)N values
print(" Choosing the range of Ne values ",quote="FALSE" )
if(XAvail) print("  note that a log_10 scale of sizes is used here",quote="FALSE" )
print(paste("  maximal  Ne  value is ",xmax,". Give an upper value:"),quote="FALSE" )
xmax= readline("  enter maximum  Ne  value= ")
print(paste("  minimal  Ne  value is ",xmin,". Give a lower value:"),quote="FALSE" )
xmin= readline("  enter minimum  Ne  value= ")
if(XAvail)
{
ymax = max(dat[cases,514:1025])
legy = "density of posterior distribution of log_10(N(T)) "
legx = " log_10(Ne)  values "
}
else
{
ymax = max(dat[cases,3:514])
legy = "density of posterior distribution of N(T) "
legx = " Ne  values "
}
#
TITRE = paste(colours[1:nbcases],": at T= ",(dat[cases,1])," in the past")
# TITRE = paste("Time in the past \n", colours[1:nbcases], "\n", (dat[cases,1])
par(adj=0.5)
plot(c(xmin,xmax),c(0,ymax),type='n',xlab=legx,ylab=legy)
par(adj=0)
title(sub= parafile,main=TITRE,cex.main=1,cex.sub=1)
#
for (ti in 1:nbcases) {
time = cases[ti]
if(XAvail) 
{
x = as.numeric(dat[time,2:513])
y = as.numeric(dat[time,514:1025])
}
else
{  
x=as.numeric(dat[time,2])
x= x*c(0:511)
y=as.numeric(dat[time,3:514])
}
lines(x,y,col=colours[ti])
}
     }
#  end of while loop
               }
return()
}
##############################################################################
#
# VarEff suite, function Tmrca()
#
# Deriving the posterior distribution of  Tmrca, the
# Time to the Most Recent Common Ancestor of two alleles,
# using a  .Batch  calculted by VarEff
#
# Since .Batch data are given in reduced scales (theta's for sizes and 
# generation times mutation rate for times, the mutation rate musr be 
# provided to show results in natural time (generations).
#
# The function allows the distribution of Tmrca and
# of log(Tmrca) to be visualized.
#
# The function includes parameters maxT and LogmaxT 
# that allow one to fix the range of past times along which
# the densities of the distribution of Tmrca is calculated.
# If provided explicitly when calling the function, a single calculation
# is performed with these values, and the ditributions are plotted.
# If one of these parameters (or both) are not provided, the function is 
# to be used interactively and it allows the user to check several ranges of time.
#
#
Tmrca <- function(batchfile=NULL,MUTAT=NULL,TMAXin=NULL,LogTMAXin=NULL)
{
# Retrieving the posterior distribution of TMRCA
print(" VarEff suite - function Tmrca()",quote="FALSE"  )
print(" Retrieving the posterior distribution of TMRCA",quote="FALSE"  )
if( !is.null(batchfile) ) { print(c(" Batch file = ",batchfile),quote="FALSE"  ) }
else {batchfile = readline(" Give the name of a .Batch file ") }
#
if( !is.null(MUTAT) ) { print(c(" Assumed mutation rate = ",MUTAT),quote="FALSE"  ) }
else {MUTAT = as.numeric( readline(" Enter the mutation rate ") ) }
#
# Read the .Batch file
Batch = as.matrix( read.table(batchfile) )
parafile = substring(batchfile,1,nchar(batchfile)-6)
#
mrcafile = paste(parafile,"Tmrca",sep=".") 
#
write(" VarEff suite, function Tmrca()",file=mrcafile)
write(paste(" job: ",parafile,", mutation rate = ",MUTAT),file=mrcafile,append="TRUE")
# Get JMAX and Nbsim
Nbsim = length( Batch[,1] )
#
# test
# Nbsim = 1000
#
JMAX = ( length( Batch[1,] ) - 5 ) /2
# Limits of theta's and T's
L1 = 4
L2 = 4 + JMAX
L3 = L2+1
L4 = L2 + JMAX
#
# Limit of number of points
# 
Nbpoints=1000
#
# Search the maximal value of T's
maxT = max( Batch[,L4] )
meanT = mean( Batch[,L4] )
stdT = sd( Batch[,L4] )
print(" Mean, St.dev and Max of T_JMAX",quote="FALSE"   )
print( c(round(meanT/MUTAT),round(stdT/MUTAT),round(maxT/MUTAT)),quote="FALSE"   )
# 
maxT0 = meanT + 2*stdT
print(c(" Most ancient proposed time =",round(maxT0/MUTAT)),quote="FALSE"  )
#
print(" Plotting the posterior density of T_MRCA",quote="FALSE"  )
# check input data
if( !is.null(TMAXin) ) 
{
print(c(" Upper limit for ancient times (generations ago) =",TMAXin),quote="FALSE"  ) 
maxT = TMAXin
}
else
{
print(" Enter upper limit for ancient times (generations ago)",quote="FALSE"  ) 
maxT = as.numeric( readline(" (type 0 to turn to Log(TMRCA) densities) TMAX = ") )  
}
maxT =maxT *MUTAT
#
    while(maxT > 0) {
#
touspoints = MUTAT + c(0:(Nbpoints-1)) * (maxT-MUTAT)/Nbpoints
#
delta = (maxT-MUTAT)/Nbpoints
#
# re-read Batch and calculate densities at all points, then cumulate
#
# Nbpoints = length(all points)
fnat = rep(0,Nbpoints)
snat = fnat
#
for (sim in 1:Nbsim)
{
T = Batch[sim,L3:L4]
Theta = Batch[sim,L1:L2]
# start at "zero"
fnat[1] = 2/Theta[1]
# the J intervals
Prod = 2
point = 2
time = touspoints[2]
# first interval 
J=1
     while( (time <= T[1]) & (point <= Nbpoints)  )
     {
      fnat[point] = Prod * exp( -2*(time-MUTAT)/Theta[1] ) / Theta[1]
      point = point + 1
      time = touspoints[point]
     }
    Prod = Prod * exp(-2*( T[J] - MUTAT)/Theta[J] )
# next ones
         if(JMAX > 1) {
  for (J in 2:JMAX)
  {
     while( (time <= T[J]) & (point <= Nbpoints)  )
     {
      fnat[point] = Prod * exp( -2*(time-T[J-1])/Theta[J] ) / Theta[J] 
      point = point + 1
      time = touspoints[point]
     }
    Prod = Prod * exp(-2*( T[J] - T[J-1] )/Theta[J] )
  }
                        }  
# the end
     while (point <= Nbpoints )
     {
      time = touspoints[point]
      fnat[point] = Prod * exp( -2*(time-T[JMAX])/Theta[JMAX+1] ) / Theta[JMAX+1]
      point = point + 1
     }
#
# cumul
snat = snat + fnat
}
# end of the Nsim loop
#
# take means
snat = snat / Nbsim
proba = round(sum(snat) * delta,3)
print(c(" Prob( TMRCA < TMAX ) = (approx)",proba) ,quote="FALSE"  )
#
write(paste(" Natural density of Tmrca, TMAX = ",round(maxT/MUTAT)),file=mrcafile,append="TRUE")
write(paste(" Prob( TMRCA < TMAX ) = (approx)",proba),file=mrcafile,append="TRUE")
# plot
# natural density
par(adj=0.5)
plot(c(0,maxT/MUTAT),c(0,max(snat)*MUTAT),type='n',xlab="Time in the past (generations)",ylab="density")
title(main="Density of TMRCA posterior distribution")
par(adj=0)
title(sub=parafile)
lines(touspoints/MUTAT,snat*MUTAT)
#
if( !is.null(TMAXin) ) { maxT=0 } 
else
{
print(" Plotting the posterior density of T_MRCA",quote="FALSE"  )
print(" Enter upper limit for ancient times (generations ago)",quote="FALSE"  ) 
maxT = as.numeric( readline(" (type 0 to turn to Log(TMRCA) densities) TMAX = ") ) 
maxT =maxT *MUTAT
}
#
             }
#
print(" Plotting the posterior density of log(T_MRCA)"  ,quote="FALSE"  )
# open new window
windows()
#
if( !is.null(LogTMAXin) ) 
{
print(c(" Upper limit for ancient times (generations ago) =",LogTMAXin),quote="FALSE"  ) 
LogmaxT = LogTMAXin
}
else
{
print(" Enter upper limit for ancient times (generations ago)",quote="FALSE"  ) 
LogmaxT = as.numeric( readline(" (stop on 0) TMAX = ") ) 
}
LogmaxT = LogmaxT * MUTAT
#
#
        while(LogmaxT > 0)  {
#

# version log:
touspoints = exp( log(MUTAT) + c(0:(Nbpoints-1)) * (log(LogmaxT)-log(MUTAT)) /Nbpoints )
# il faut refaire le calcul en ces points
#
# re-read Batch and calculate densities at all points, then cumulate
#
# Nbpoints = length( touspoints)
flog = rep(0,Nbpoints)
slog = flog
#
for (sim in 1:Nbsim)
{
T = Batch[sim,L3:L4]
Theta = Batch[sim,L1:L2]
# start at "zero"
flog[1] = 2/Theta[1]* MUTAT
# the J intervals
Prod = 2
point = 2
time = touspoints[2]
# first interval 
J=1
     while( (time <= T[1]) & (point <= Nbpoints)  )
     {
      flog[point] = time * Prod * exp( -2*(time-MUTAT)/Theta[1] ) / Theta[1]
      point = point + 1
      time = touspoints[point]
     }
    Prod = Prod * exp(-2*( T[J] - MUTAT)/Theta[J] )
# next ones
    if (JMAX>1)   {
  for (J in 2:JMAX)
  {
     while( (time <= T[J]) & (point <= Nbpoints)  )
     {
      flog[point] = time * Prod * exp( -2*(time-T[J-1])/Theta[J] ) / Theta[J]
      point = point + 1
      time = touspoints[point]
     }
    Prod = Prod * exp(-2*( T[J] - T[J-1] )/Theta[J] )
  }
                   }
# the end
     while (point <= Nbpoints )
     {
      time = touspoints[point]
      flog[point] = time * Prod * exp( -2*(time-T[JMAX])/Theta[JMAX+1] ) / Theta[JMAX+1]
      point = point + 1
     }
# cumul
slog = slog + flog
}
# end of the Nsim loop
#
# take means
slog = slog / Nbsim
proba= round( sum(slog)*(log(LogmaxT)-log(MUTAT)) /Nbpoints ,3)
#
write(paste(" Density of log(Tmrca), TMAX = ",round(LogmaxT/MUTAT)),file=mrcafile,append="TRUE")
write(paste(" Prob( TMRCA < TMAX ) = (approx)",proba),file=mrcafile,append="TRUE")
#
print(c("  Prob( TMRCA < TMAX ) = (approx) ",proba),quote="FALSE"   )
# plot
par(adj=0.5)
plot(c(0,log10(LogmaxT/MUTAT)),c(0,max(slog)*log(10)),type='n',xlab="Time in the past ( log(generations) )",ylab="density")
title(main="Density of log(TMRCA) posterior distribution")
par(adj=0)
title(sub=parafile)
lines(log10(touspoints/MUTAT),slog*log(10))
#
if( !is.null(LogTMAXin) ) { LogmaxT=0 } 
else
{
print(" Plotting the posterior density of log(T_MRCA)"  ,quote="FALSE"  )
print(" Enter upper limit for ancient times (generations ago)",quote="FALSE"  ) 
LogmaxT = as.numeric( readline(" (stop on 0) TMAX = ") ) 
LogmaxT = LogmaxT * MUTAT
}
#
  }
#             }            
return()
# end of function Tmrca
}
# end of function Tmrca
##############################################################################
##############################################################################
# # Internal functions with explicit transfer of parameters
##############################################################################
# Estimation of the likelihood of thetatau (parameters N_i and differences  t_(j) - t_(j-1)  )
.Logvraisembl <- function(thetatau,JMAX,observed)
{
DMAXPLUS = observed$DM
espfreqdis <- vector(mode="numeric",length=DMAXPLUS)
# Means of theorics fk
  for (dista in 1:DMAXPLUS)
 { 
   intint = integrate(.fonvarjmax2 ,0,pi,dista-1,JMAX,thetatau,stop.on.error = FALSE)
   if(intint$message == "OK"){espfreqdis[dista] <- intint$value/pi}else{return(-Inf)}
  }
# Vector of VarCovInv * espfreqdis
vciprod <- observed$IV %*% espfreqdis
return(observed$NL*sum(vciprod*(2*observed$FR-espfreqdis))/2)
}
############################################################################## 
# Posterior probability of parameters
.Laposter <- function(parametre,JMAX,observed,priors)
{
COEF1 = priors$C1
COEF2 = priors$C2
COEF3 = priors$C3
RHOCORN = priors$RO
thetatau <- priors$TZ * exp(parametre)
# check for c < 1
if( abs(thetatau[2*JMAX+2]) > .99) return(-Inf)
valeur <- .Logvraisembl(thetatau,JMAX,observed)
if(valeur == -Inf) return(-Inf)
# 
 ap1 <- COEF2*sum(parametre[(JMAX+2):(2*JMAX+1)]^2)
 ap2 <- sum(parametre[1:(JMAX+1)]^2)
 ap2 = ap2 + RHOCORN *(RHOCORN * sum(parametre[2:JMAX]^2)
     - 2 * sum(parametre[1:JMAX] * parametre[2:(JMAX+1)]))
#
# add the term for "C"
return(valeur + ap1 + COEF1 * ap2 + COEF3 * parametre[2*JMAX+2]^2)
}
##############################################################################
# Function submitted to the Fourier inversion, to get expected values of the fk
.fonvarjmax2  <- function(x,dista,JMAX,thetatau)
{
        qmuc <- 1 - .geom(x,thetatau[2*JMAX+2])
        jplus <- JMAX + 1
        valeur <- 1/(1 + thetatau[jplus] * qmuc)
        j <- JMAX 
        jplus <- jplus + JMAX 
        {
                while(j > 0) {
                        ajet <- 1 + thetatau[j] * qmuc
                        beta <- exp( - thetatau[jplus] * ajet)
                        valeur <- valeur * beta + (1 - beta)/ajet
                        j <- j - 1
                        jplus <- jplus - 1
                }
        }
        return(valeur * cos(dista * x))
}
##############################################################################
#Goemetric function
#TPM, C < 0
.geom <- function(x,c)
{
cosx = cos(x)
if( c == 0 ) return(cosx)
#
if( c < 0 ) return((1+c)*cosx - c * cos(2*x))
#
a = cosx - c
ac= c*a
return((a-ac)/(1-ac-ac-c*c))
}
##############################################################################
# Matrix function to set the scale matrix
.prodEE <- function(j,V1,V2,V3)
{
j1 <- j +1
j2 <- j +2
Nb <- 2*j1
 param1 = rep(0,Nb)
 param2 <- param1
 param3 <- param1
param1[1:j1]      = 1
param2[j2:(Nb-1)] = 1
param3[Nb]        = 1
return( V1* param1 %*% t(param1)
        + V2 * param2 %*% t(param2) 
        + V3 * param3 %*% t(param3))
}
##############################################################################
# Matrix function to set the scale matrix
.vdmdiretendue <- function(rho,jm)
 {
 djmp <- 2*jm+2
vander = diag(rep(1,djmp))
for (i in 1:jm) {
  for (j in (i+1):(jm+1)) {
  vander[j,i] <- rho^(j-i) 
  vander[i,j] = vander[j,i]
  }
}
 return(vander) 
}
##############################################################################
# Calculates the likelihoods and posterior probabilities of the series (Iterations) 
# of results tt = out$batch
.distriL <- function(tt,JMAX,observed,priors)
{
Nbsim <- length(tt[,1])
distriLike <- vector(mode="numeric",length=Nbsim)
distriPost <- vector(mode="numeric",length=Nbsim)
   for (simu in 1:Nbsim) {
    Lapos2 = .Laposter2(tt[simu,],JMAX,observed,priors)
    distriLike[simu] <- Lapos2$L
    distriPost[simu] <- Lapos2$P
                         }
return(list(L=distriLike,P=distriPost))
}
##############################################################################
# Return the likelihoods and posterior probabilities of parameters
.Laposter2 <- function(parametre,JMAX,observed,priors)
{
COEF1 = priors$C1
COEF2 = priors$C2
COEF3 = priors$C3
RHOCORN = priors$RO
thetatau <- priors$TZ * exp(parametre)
# check for c < 1
if( thetatau[2*JMAX+2] > .99 ) return( -Inf )
valeur <- .Logvraisembl(thetatau,JMAX,observed)
# Fixe the COEF2 <- 1/(2*VARP2)  and  COEF1 <- - 1 /( 2* VARP1 *(1- RhoCorN^2))
ap1 <- COEF2*sum(parametre[(JMAX+2):(2*JMAX+1)]^2)
 ap2 <- sum(parametre[1:(JMAX+1)]^2)
 ap2 = ap2 + RHOCORN*(RHOCORN*sum(parametre[2:JMAX]^2)
     - 2*sum(parametre[1:JMAX]*parametre[2:(JMAX+1)]))
#
# add term for "C"
valpost = valeur + ap1 + COEF1 * ap2 + COEF3 * parametre[2*JMAX+2]^2 
return(list(L=valeur,P=valpost))
}
##############################################################################
# Theoretical approximate value of the diagonal of the variance covariance matrix of the fk
.diagtheo <- function(theta,kmax)
{
Expa = 0.05307618
Bthe = 1.1407126
coefth = exp(-2/sqrt(1+2*theta))
dth = c(1:kmax)
dth[1] = Expa/(theta**Bthe)
if( kmax > 1 ) {
for (k in 2:kmax)
   {
   dth[k] = dth[k-1] * coefth
   }
               }
return(dth)
}
##############################################################################
# From "out$batch" of metrop, extraction of natural parameters
# input : tt = out$batch
#         prio = prior values of theta and tau parameteres 
#                (units 4Nu and g / 2N and  c_0 )
# output
# "tt" is the transformed result from "out$batch" where
#    log parameters (1:JMAX+1) are returned to their natural scale (theta)
#    and parameters (JMAX+2:2*JMAX+2) are transformed from log-intervals
#                                     to tansitions times (scale g_i u)
#
# suppression du N_0 si t_1 < une generation, ie t_1 < MUTAT
#             en imposant theta_0 = theta_1
#
.ExtracNatural <- function(tt,JMAX,prio,MUTAT)
{
 Nbsim <- length(tt[,1])
for (sim in 1:Nbsim) {tt[sim,] <- prio * exp( tt[sim,])}
# Units = generation * mutation
 for (sim in 1:Nbsim) {
   gmu <- 0
            for  (j in 1:JMAX) {   
            gmu <- gmu + 0.5 * tt[sim,(JMAX+1+j)] * tt[sim,j]
            tt[sim,(JMAX+1+j)] <- gmu 
                               
# overall correction N_(i+1)  -->  N_(i)  if  T_(i) < MUTAT
for (jt in (2*JMAX+1):(JMAX+2))
{
if(tt[sim,jt] < MUTAT) tt[sim,(jt-JMAX-1)] = tt[sim,(jt-JMAX)] 
}                              }
                      }
return(tt)
}
##############################################################################
# From "out$batch" of metrop, extraction of the distribution of N(t ) 
# for a set of  analysed times (NbInstants)
# "tt" is the transformed result from "out$batch" where
#    log parameters (1:JMAX+1) are returned to their natural scale (theta)
#    and parameters (JMAX+2:2*JMAX+2) are transformed from log-intervals
#                                     to tansitions times (scale g_i u)
# This transformation is performed by function ExtracNatural(...)
#           
.ExtracTheta <- function(tt,TempsMax,Nbinstants,JMAX)
{
 Nbsim <- length(tt[,1])
#Distribution of theta in function of analysed times (NbInstants)
Longint <- TempsMax / Nbinstants
Kmax <- Nbinstants + 1
ThetaK <- matrix(nrow=Nbsim,ncol=Kmax)
ThetaK[,1] <-  tt[,1]
deltak <- 0
{ for (k in 2:Kmax) 
  {  deltak <- deltak + Longint
      { for (sim in 1:Nbsim) 
         {  
Nbdep <-1
{ for (j in 1:JMAX) 
  { if( deltak >=  tt[sim,(JMAX+1+j)] ) Nbdep <- Nbdep + 1 }
}
             ThetaK[sim,k] <- tt[sim,Nbdep]   }
       }
  }
}
# Mean 
# July 2011 "ecartype" is used to keep harmonic means
moyennes <- vector(mode="numeric",length=Kmax)
 ecartype <- vector(mode="numeric",length=Kmax)
#
# V2, July 2013, (re)introducing standard deviations.
 stadev <- vector(mode="numeric",length=Kmax) 
#
for  (k in 1:Kmax)
{ 
moyennes[k]  <- mean( ThetaK[,k])
ecartype[k]  <- 1 / mean( 1 / ThetaK[,k]) 
stadev[k]   <- sd( ThetaK[,k])
}
# Sample times 
TempEch <- vector(mode="numeric",length=Kmax)
  deltak <- 0
  TempEch[1] <- 0
 { for (k in 2:Kmax){ deltak <- deltak + Longint
 TempEch[k] <- deltak }}
# Quantiles
 QuantP3 <- matrix(nrow=Kmax,ncol=5)
 {for (k in 1:Kmax) {
  QuantP3[k,] <- quantile(ThetaK[,k],probs=c(0.01,0.05,0.5,0.95,0.99),type=4)
 }}
return( list(tK=ThetaK,moy=moyennes,eca=ecartype,tech=TempEch,Q=QuantP3,stade=stadev) )
}
##############################################################################
# From trajectories  {t - N} in natural scales, table tt,
# "tt" is the transformed result from "out$batch" where
#    log parameters (1:JMAX+1) are returned to their natural scale (theta)
#    and parameters (JMAX+2:2*JMAX+1) are transformed from log-intervals
#                                     to tansitions times (scale g_i u)
# This transformation is performed by function ExtracNatural(...)
#  
# build a map, a Ntime * Nsize  matrix in which element (i,j) is
# the density of the joint (T,N) distribution
#
# range is  0-TempsMax for times
# the range for sizes is adjusted, censoring highest and lowest sizes of proba < 0.10
#
# data :  tt, TempsMax, Ntime, Nsize, JMAX
# output: map density, in log scale (natural log) for theta's
#         
.LogMap <- function(tt,TempsMax,Ntime, Nsize,JMAX)
{
Nbsim <- length( tt[,1] )
map = matrix(0,nrow=Ntime,ncol=Nsize)
deltaTime = TempsMax/Ntime
# calculate upper size
maxiz = vector(mode="numeric",length=Nbsim)
for (sim in 1:Nbsim) { maxiz[sim] = max( tt[sim,1:(JMAX+1)] ) }
miniz = vector(mode="numeric",length=Nbsim)
for (sim in 1:Nbsim) { miniz[sim] = min( tt[sim,1:(JMAX+1)] ) }
SizeMax = as.numeric( quantile( maxiz, probs=c(0.95), type=4 ) )
SizeMin = as.numeric( quantile( miniz, probs=c(0.05), type=4 ) )
#
deltaSize = log( SizeMax/SizeMin )/Nsize
#
Limit = vector(mode="numeric",length=JMAX)
#
proba = 1
#
for (sim in 1:Nbsim) {
# identify times of transition 
  for (j in 1:JMAX)  {
     Lim      = as.integer( tt[sim,(JMAX+1+j)] / deltaTime ) 
     Limit[j] = min( Lim , Ntime )
                    }
#
# fill map 
# first interval
  if( Limit[1] >0 )
    {
    ordo = as.integer( log( tt[sim,1]/SizeMin ) / deltaSize )
    if( ((ordo+1)>0) & (ordo<Nsize) ) {
    ordo = ordo + 1
    for (ell in 1:Limit[1] ) 
       { map[ell,ordo] = map[ell,ordo] + proba }
                   }
    }
# 2nd to (JMAX)th intervals (provided JMAX > 1)
             if ( JMAX > 1 ) {
  for (j in 2:JMAX) {
       if( Limit[j] > Limit[j-1] )
          {
           ordo =  as.integer( log( tt[sim,j]/SizeMin) / deltaSize )
           if(((ordo+1)>0) & (ordo<Nsize) ) {
           ordo = ordo + 1
           for (ell in (Limit[j-1]+1):Limit[j] ) 
             { map[ell,ordo] = map[ell,ordo] + proba }
                          }
          }
                    }
                                                }
# last interval
   if( Limit[JMAX] < (Ntime-1) )
     {
      ordo =  as.integer( log( tt[sim,(JMAX+1)]/SizeMin) / deltaSize )
      if(((ordo+1)>0) & (ordo<Nsize) ) {
      ordo = ordo + 1
      for (ell in (Limit[JMAX]+2):Ntime )
         { map[ell,ordo] = map[ell,ordo] + proba }
                       }
    }
############    end "sim" loop
}
return( list( M=map,SM=log(SizeMax),sm=log(SizeMin) ) )
}
##############################################################################
#
# read data and extract allele frequencies
#           skipping loci with 0 sample size
#
# input: infile, data file
#          NSIM,  number of repetitions (simulations)
#          NBLOC, number of markers
#
# output: NbMaxAll, PkMoyTout, VarCovTout
#          and the effective number of loci: NELOC
#
.DataExtract <- function(infile,NBLOC)
{
# Read the data file
#
TOUT <- scan(infile)
#

#
# First step: derive the maximum number of  alleles "NbMaxAll" 
#             from one or several samples of the population: 
#
NbMaxAll = 1
posit = 0
#
# Analyse each sample (simu)
#

# Look the number of alleles at each markers 
for (locus in 1:NBLOC) {
posit = posit +1
nbal = TOUT[posit]
# Find the allele max
NbMaxAll = max(NbMaxAll,nbal)
# Step to the next marker 
posit = posit + nbal
}
# End
# 
# Second step: Extracting frequencies
#
# Read again
posit = 0
Genot = vector(mode="numeric",length=NbMaxAll)
PkMoy = vector(mode="numeric",length=NbMaxAll)
FreqDis = matrix(nrow=NBLOC,ncol=NbMaxAll)
VarCov = matrix(nrow = NbMaxAll,ncol=NbMaxAll)
# 
# Save the number of alleles at each markers "Nall[locus] = nbal"
#
locut = 0
	for (locus in 1:NBLOC) {
	posit = posit +1
	nbal = TOUT[posit]
	Genot = Genot * 0
   		for (al in 1:nbal) {
   		posit = posit + 1
   		Genot[al] = TOUT[posit]
   		}
# Frequencies
   		nbgenes = sum( Genot[] )
# check sample size
		if( nbgenes > 2 )  {
#
		locut = locut + 1
		if ( nbgenes < 10 ) {
 		print(c(" Marker ",locus," , low sample size = ",nbgenes), quote="FALSE")
					   }
#
 		nbgen2  = sum( Genot[]**2 )
# FreqDis(0) = (nbgen2 - nbgenes) / ( nbgenes*(nbgenes-1) )
		FreqDis[locut,1] = nbgen2 - nbgenes
# Initialization
		FreqDis[locut,2:NbMaxAll] = 0
# Browse the pairs of alleles  al1, al2 
  		if( nbal > 1 )  {
   			for (al1 in 1:(nbal-1) ) { for (al2 in (al1+1):nbal ) {
   			delta = al2-al1+1
   			FreqDis[locut,delta] =FreqDis[locut,delta] + Genot[al1]*Genot[al2]
   			}}
            	}
# Standardization
		FreqDis[locut,] = FreqDis[locut,] / ( nbgenes*(nbgenes-1) )
}
					}
# End
if ( locut < NBLOC ) FreqDis[1:locut,] -> FreqDis
# Extracting PkMoy, VarCov
for (delta in 1:NbMaxAll ) { PkMoy[delta] <- mean( FreqDis[,delta] ) }
 VarCov = var( FreqDis )
return( list(L=locut,A=NbMaxAll,P=PkMoy,V=VarCov) )
}
############################################################################## 
# function to write parameters to a file
.parDiagwrite  <- function(parafile,infile,NBLOC,JMAX,MODEL,MUTAT,NBAR,VARP1,RHOCORN,GBAR,VARP2,DMAXPLUS,Diagonale,NumberBatch,LengthBatch,SpaceBatch,Burnin,AccRate)
#
{
#parval = vector(length=18)
#
parval=c(parafile,infile,NBLOC,JMAX,MODEL,MUTAT,NBAR,VARP1,RHOCORN,GBAR,VARP2,DMAXPLUS,Diagonale,NumberBatch,LengthBatch,SpaceBatch,Burnin,AccRate)
#tete = vector(length=18)
tete = c("VarEff(parafile = ","infile = ","NBLOC = ","JMAX = ","MODEL = ",
"MUTAT = ","NBAR = ",
"VARP1 = ","RHOCORN  = ","GBAR = ","VARP2 = ","DMAXPLUS = ",
"Diagonale = ",
"NumberBatch = ","LengthBatch = ","SpaceBatch = ","Burnin = ","AccRate = ")
#
ligne = vector(length=18)
ligne[c(1:2,5)]=paste(tete[c(1:2,5)],"'",parval[c(1:2,5)],"'",",",sep="")
ligne[c(3:4,6:17)]=paste(tete[c(3:4,6:17)],parval[c(3:4,6:17)],",",sep="")
ligne[18]=paste(tete[18],parval[18],")",sep="")
ecrit= matrix(ligne,nrow=length(ligne))
write(ecrit,ncolumns=1,file=paste(parafile,".R",sep=""))
}
#
############################################################################## 
# read the mutation model
.MutMod <- function(Answer)
{
Coef = 0
rep = .extrTC(Answer)
if( rep$A == "S" ) return(Coef)
if( rep$A == "G" ) Coef = rep$C
if( rep$A == "T" ) Coef = - rep$C
return(Coef)
}
.extrTC <- function(x)
{
#extract first  non blank character
while(substring(x,1,1) == " ") {x = substring(x,2) }
Rep = substring(x,1,1)
if(Rep == 0) {Coef=0} else {Coef=as.numeric(substring(x,2))}
return(list(A=Rep,C=Coef))
}
############################################################################## 
# Adjust "scalecal" to get the chosen acceptation rate of MCMC / V2
# 
# Input :  the posterioir prob function Lapo
#          paraminit
#          JMAX and the lists observed and priors
#          the requested acceptation rate:  refacc
#          the firts value of the proposal scalecal
# Output:  $SC = the new scalecal value
#          $OO = the last "out" product from metrop
.Refa2 <- function(Lapo,paraminit,JMAX,observed,priors,refacc, scalecal)
{
# refacc = 0.25
#test
#print(paraminit)
#print(scalecal)
#print(JMAX)
out <- metrop(Lapo,paraminit,nbatch=1000,blen=1,nspac=1,scale=scalecal,,,JMAX,observed,priors)
acc1 = out$accept
acc2 = 0.0
coef = 0.0
#test
#print(acc1)
if( abs(acc1-refacc) > 0.02 )
{
if (acc1 < refacc){coef=0.2}else{coef=5}
scalecal = scalecal * coef
out <- metrop(out,scale= scalecal,,,,,,,,JMAX,observed,priors)
acc2 = out$accept
#test
#print(acc2)
#
# Linear iteration 
while ( abs(acc2-refacc) > 0.02 )
{
# print( paste(acc1,acc2,coef) )
if ( abs(acc1-acc2) > 0.01 )
{ 
   coef = ( (refacc-acc2) / coef + acc1 - refacc )/(acc1-acc2) 
   if( coef < 0 ) coef = 0.5
   if( coef > 5 ) coef = 5.0
}
else { if (acc1 < refacc){coef=0.2}else{coef=5} }
scalecal = scalecal * coef
out <- metrop(out,scale= scalecal,,,,,,,,JMAX,observed,priors)
acc1 = acc2
acc2 = out$accept
#test
#print(acc2)
}
}
return(list(SC=scalecal,OO=out))
}
############################################################################## 
# function .LSD(NameBATCH=NULL,TMAX=NULL,NBT=NULL,MUTAT=NULL,optionlog)
# commmon function called by  NatSizeDist() and LogSizeDist()
.LSD <- function(NameBATCH,TMAX,NBT,MUTAT,optionlog)
{
#
parafile = substring(NameBATCH,1,nchar(NameBATCH)-6)
#
BATCH = read.table(NameBATCH)
#
Lenfull = length(BATCH[1,])
JMAX    = (Lenfull-3)/2 - 1
tt      =  BATCH[,4:Lenfull]
#
# scaling according to MUTAT
# on entry, TMAX is in reduced scale G*MUTAT
TMAXplot = TMAX
if(!(MUTAT == 0) )
{
TMAXplot = TMAX /MUTAT
tt[,1:(JMAX+1)] = tt[,1:(JMAX+1)] /(4*MUTAT)
tt[,(JMAX+2):(2*JMAX+1)] = tt[,(JMAX+2):(2*JMAX+1)] / MUTAT
}
distriresu = matrix(0,nrow=(NBT+1),ncol=514)
#
#  option LOG : passage en log_10(theta ou N)
if(optionlog) 
{
tt[,1:(JMAX+1)] = log( tt[,1:(JMAX+1)] )/log(10)
distriresu = matrix(0,nrow=(NBT+1),ncol=1025)
}
#
# call ExtracTheta
extrait = .ExtracTheta(tt,TMAXplot,NBT,JMAX)
#
# V3 write the full extrait$tK matrix to file
# providing the nbatch values of N at all the selected times
# first line gives these times TMAXplot*c(0:NBT)/(NBT) 
# first column gives the number of the simulation
nbatch = length(BATCH[,1])
outfile = paste(parafile,".Nsize",sep="")
if(optionlog) outfile = paste(parafile,".Lsize")
tKN = matrix(0,nrow=(nbatch +1),ncol=(NBT+2))
tKN[,1] = c(0:nbatch )
tKN[2:(nbatch +1),2:(NBT+2)] = extrait$tK 
tKN[1,2:(NBT+2)] = TMAXplot*c(0:NBT)/(NBT) 
write(t(tKN),ncolumns=(NBT+2),file=outfile)
#
Nbresu = 0
# V2 : extend length of Resus
 Resus  <- rep(0, (7*(NBT+1)) ) 
 for   (tem in 1:(NBT+1))
{
Nbresu = Nbresu+1
Resus[Nbresu] = extrait$moy [tem]
Nbresu = Nbresu+1
if(!optionlog) Resus[Nbresu] = extrait$eca[tem]
Nbresu = Nbresu+1
#
# Mode estimates by density function
liminf = 0
if(optionlog) liminf = extrait$Q[tem,1]
#  densNtem <- density(extrait$tK[,tem],from=liminf,to=extrait$Q[tem,5],adjust=0.5)
densNtem <- density(extrait$tK[,tem],from=liminf,to=extrait$Q[tem,5])
mode = densNtem$x[which.max(densNtem$y)]
#
# save the density
if(optionlog)
{
distriresu[tem,2:513]= densNtem$x[]
distriresu[tem,514:1025]=densNtem$y[]
}
else
{
distriresu[tem,2]= (densNtem$x[512]-liminf) / 511
distriresu[tem,3:514]=densNtem$y[]
}
#
Resus[Nbresu]=mode
#
Nbresu=Nbresu+1
Resus[Nbresu]=extrait$Q[tem,3]
Nbresu=Nbresu+1
Resus[Nbresu]=extrait$Q[tem,2]
Nbresu=Nbresu+1
Resus[Nbresu]=extrait$Q[tem,4]
# V2, adding standard deviation
Nbresu=Nbresu+1
Resus[Nbresu]=extrait$stade[tem]
}
#
#############################################
# Outputs
#
# Collect the results from "Resus" to give some global values 
# and to built some tables in function of "Tempsmax" (-> TMAX)
#
# Mean, mode and median of the effective size in function of the time
#
nbtemps  = length(Resus[])/7
meanmoy= vector(mode="numeric",length=nbtemps)
meanmod= vector(mode="numeric",length=nbtemps)
meanmed= vector(mode="numeric",length=nbtemps)
#
meanhar= vector(mode="numeric",length=nbtemps)
#
 {for (tem in 1:nbtemps) meanmoy[tem] = Resus[1+7*(tem-1)]}
#
 {for (tem in 1:nbtemps) meanmod[tem] = Resus[3+7*(tem-1)]}
#
 {for (tem in 1:nbtemps) meanmed[tem] = Resus[4+7*(tem-1)]}
#
 {for (tem in 1:nbtemps) meanhar[tem] = Resus[2+7*(tem-1)]}
#
# File .Nstat summary statistics of the N(T)
# 
sortieresu = matrix(ncol=8,nrow=nbtemps)
#
# adjust size scale to MUTAT
#
legabs = " Time T in the past (unit T= G*u) "
legord = " Theta(T)"
if(optionlog) legord = " log_10( Theta(T) )"
if(!(MUTAT == 0)) 
{ 
legabs = " Time G in the past (generations)"
legord = " Ne(G)"
if(optionlog) legord = " log_10( Ne(G) )"
}
#
# V2 : adding an 8th column
for (tem in 1:nbtemps) 
{
sortieresu[tem,1]=TMAXplot*(tem-1)/(NBT)
for (j in 1:7) {sortieresu[tem,(j+1)]=Resus[(j+7*(tem-1))]}
}
#
if(optionlog)
{ write(t(sortieresu),ncolumns=8,file=paste(parafile,"Lstat",sep=".")) }
else
{ write(t(sortieresu),ncolumns=8,file=paste(parafile,"Nstat",sep=".")) }
#
print(" Plot of mean results ",quote="FALSE")
#
nbtm = nbtemps-1
echelle = TMAXplot/nbtm
 topordo = max(meanmoy,meanmod,meanmed,meanhar)+ 0.1
 botordo = 0
 if(optionlog) botordo = min(meanmoy,meanmod,meanmed)- 0.1
#
par(adj=0.5)
 plot(c(0,TMAXplot),c(botordo,topordo),type='n',xlab=legabs,ylab=legord)
#
par(adj=0)
if(!optionlog) title(sub=parafile,main="Mean (red), Mode (blue), Median (black) and Harmonic mean (orange)",cex.main=1,cex.sub=1)
if(optionlog) title(sub=parafile,main="Mean (red), Mode (blue) and Median (black) of log-sizes ",cex.main=1,cex.sub=1)
#
 lines(c(0:(nbtemps-1))*echelle,meanmod,col='blue')
 lines(c(0:(nbtemps-1))*echelle,meanmed,col='black')
 lines(c(0:(nbtemps-1))*echelle,meanmoy,col='red')
if(!optionlog)  lines(c(0:(nbtemps-1))*echelle,meanhar,col='orange')
#
# File .Ndist
# the densities of the posterior N(T) distributions at the chosen times
#
for (tem in 1:nbtemps) 
{
distriresu[tem,1]=TMAXplot*(tem-1)/(NBT) 
}
if(optionlog)
{
write(t(distriresu),ncolumns=1025,file=paste(parafile,"Ldist",sep="."))
# save the plot
#dev.copy2pdf(file=paste(parafile,"_LLL.pdf",sep="") )
#return()
}
else
{
write(t(distriresu),ncolumns=514,file=paste(parafile,"Ndist",sep="."))
# save the plot
#dev.copy2pdf(file=paste(parafile,"_mmm.pdf",sep="") )
#return()
}
return()
}
